@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2013 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2015 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@ from Mailman import mm_cfg
 from Mailman import Utils
 from Mailman import LockFile
 from Mailman.Message import UserNotification
+from Mailman.Bouncer import _BounceInfo
 from Mailman.Bouncers import BouncerAPI
 from Mailman.Queue.Runner import Runner
 from Mailman.Queue.sbcache import get_switchboard
@@ -151,6 +152,14 @@ class BounceMixin:
         try:
             op, addr, bmsg = mlist.pend_confirm(token)
             info = mlist.getBounceInfo(addr)
+            if not info:
+                # info was deleted before probe bounce was received.
+                # Just create a new info.
+                info = _BounceInfo(addr,
+                                   0.0,
+                                   time.localtime()[:3],
+                                   mlist.bounce_you_are_disabled_warnings
+                                   )
             mlist.disableBouncingMember(addr, info, bmsg)
             # Only save the list if we're unlocking it
             if not locked:
